#!/usr/bin/env python

import paramiko
import time

__author__ = "David Johnnes"
__email__ = "david.johnnes@gmail.com"

"""
This module can be used to establish SSH connection to any devices that supports SSH.

    For single command execution, you should use the "send_single_command() function"
    For multiple commands execution you should use the send_multiple_commands() function.
"""


def send_single_command(hostname, user_name, pass_word, command,
                        key=False, timeout=10, port=22, sleep_time=6):
    """ This function will be used to send a single to a remote machine.

    """

    ssh = paramiko.SSHClient()

    try:
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=hostname, port=port, username=user_name,
                    password=pass_word, look_for_keys=key, timeout=timeout)
        print("Connected to {0}\n".format(hostname))

    except:
        print("Could not connect to {0}".format(hostname))
        ssh.close()
    else:
        channel = ssh.invoke_shell()

        channel.send(command + "\n")
        time.sleep(2)

        output = channel.recv(9999)
        time.sleep(sleep_time)

        print(output)
        print("#" * 120)

        channel.close()
        ssh.close()
        return output


def send_multiple_commands(hostname, user_name, pass_word, list_of_commands,
                           key=False, timeout=10, port=22, sleep_time=2):
    """ This function will be used to send multiple commands to  a remote machine.

    """

    ssh = paramiko.SSHClient()

    try:
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=hostname, username=user_name,
                    password=pass_word, look_for_keys=key, timeout=timeout, port=port)
        print("Connected to {0}\n".format(hostname))

    except:
        print("Could not connect to {0}".format(hostname))
        ssh.close()
    else:
        channel = ssh.invoke_shell()
        all_outputs = str()

        for cmd in list_of_commands:

            channel.send(cmd + "\n")
            time.sleep(2)

            output = channel.recv(99999)
            time.sleep(sleep_time)

            all_outputs += output
            print(output)
            print("#" * 120)
            print("\n")

        channel.close()
        ssh.close()
        return all_outputs






from juraSSH import parallel


def run_multiple_cmds(username, password, commands, hosts_file, port=22, key=False, timeout=10, sleep_time=6):
    """
    This function allows you to run a multiple commands on multiple remote machines
        in parallel
    """

    parallel.USERNAME = username
    parallel.PASSWORD = password
    parallel.COMMAND = commands
    parallel.HOSTS_FILE = hosts_file

    parallel.KEY = key
    parallel.PORT = port
    parallel.TIMEOUT = timeout
    parallel.SLEEP_TIME = sleep_time
    parallel.selector(parallel.send_multiple_commands)



from juraSSH import parallel


def run_one_cmd(username, password, command, hosts_file,port=22, key=False, timeout=10, sleep_time=6):
    """ This function allows you to run a single command on multiple remote machines
        in parallel.
    """

    parallel.USERNAME = username
    parallel.PASSWORD = password
    parallel.COMMAND = command
    parallel.HOSTS_FILE = hosts_file
    parallel.KEY = key
    parallel.PORT = port
    parallel.TIMEOUT = timeout
    parallel.SLEEP_TIME = sleep_time
    parallel.selector(parallel.send_one_command)






#!/usr/bin/env python

import paramiko
import time
from multiprocessing import Pool
from juraSSH import file_handler as fh

__author__ = "David Johnnes"
__email__ = "david.johnnes@gmail.com"

"""
    This module allows you to send a single command to multiple remote host in parallel or
    multiple commands to multiple hosts in parallel.
    ================================================

    *You can create custom function by importing this module.
    *If you do not want to create you own functions , you can use the oneCDM or the multipleCDMs function,
    for they are already completed.
"""

USERNAME = ""
PASSWORD = ""

HOSTS_FILE = []
COMMAND = ""
COMMANDS = []

PORT = 22
KEY = False

TIMEOUT = 10
SLEEP_TIME = 8


def send_one_command(hostname):
    """
    This function allows you to run a single command on multiple remote machines in parallel
    """
    ssh = paramiko.SSHClient()
    try:
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        ssh.connect(hostname=hostname, username=USERNAME, password=PASSWORD,
                    port=PORT, look_for_keys=KEY, timeout=TIMEOUT)
        print("\nConnected to == >> {0}\n".format(hostname))
    except:
        fh.file_writer(hostname)    # If a host is unreachable, it will be recorded in a file.
        ssh.close()
    else:
        channel = ssh.invoke_shell()
        channel.send(COMMAND + "\n")
        time.sleep(4)

        output = channel.recv(9999)
        time.sleep(SLEEP_TIME)

        channel.send("exit\n")

        channel.close()
        ssh.close()
        print(output)
        print("#" * 120)


def send_multiple_commands(hostname,):
    """
    This function allows you to run multiple commands on multiple remote machines in parallel
    """

    ssh = paramiko.SSHClient()

    try:
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(hostname=hostname, username=USERNAME,
                    password=PASSWORD, look_for_keys=KEY, timeout=TIMEOUT, port=PORT)
        print("Connected to {0}\n".format(hostname))

    except:
        fh.file_writer(hostname)  # If a host is unreachable, it will be recorded in a file.
        ssh.close()
    else:
        channel = ssh.invoke_shell()
        channel_data = str()

        for cmd in COMMANDS:

            channel.send(cmd + "\n")
            time.sleep(2)

            output = channel.recv(99999)
            time.sleep(SLEEP_TIME)

            channel_data += output
            channel_data += "\n"*4

        print(channel_data)
        print("#" * 120)
        print("\n")

        channel.close()
        ssh.close()
        return channel_data


def selector(func_name, cores=25):
    """
     This function is used to select the function you would like to execute.
     """
    started = time.asctime()

    p = Pool(cores)
    p.map(func_name, HOSTS_FILE)
    p.close()
    p.join()

    print("\nProcessing completed !!\n".upper())
    print("Started at: {0}".format(started))
    print("Ended   at: {0}".format(time.asctime()))
    print("\nPLEASE RUN THE [ls] COMMAND TO SEE THE FILE WITH ALL UNREACHABLE HOSTS\n")


